@@ -4,7 +4,7 @@ Provides the Distribution class, which represents the module distribution
 being built/installed/distributed.
 """
 
-__revision__ = "$Id: dist.py 73836 2009-07-04 03:01:33Z tarek.ziade $"
+__revision__ = "$Id: dist.py 75000 2009-09-21 13:59:07Z tarek.ziade $"
 
 import sys, os, re
 
@@ -354,7 +354,7 @@ Common commands: (see '--help-commands' for more)
         parser = ConfigParser()
         for filename in filenames:
             if DEBUG:
-                self.announce("  reading", filename)
+                self.announce("  reading %s" % filename)
             parser.read(filename)
             for section in parser.sections():
                 options = parser.options(section)
